@@ -15,6 +15,7 @@ default_sources = [
   ("calendar", "Calendar"),
   ("todo", "Tasks"),
   ("memo", "Notes"),
+  ("calendar+todo", "Calendar/Tasks"),
 ]
 
 backend_aliases = {
@@ -25,33 +26,46 @@ backend_aliases = {
   "maemo-notes": "memo",
 }
 
+template_names = {
+  "Google": "Google Contacts",
+}
+
 calendar_names = {
   "cal_ti_calendar_private": "Private",
   "cal_ti_smart_birthdays": "Birthdays",
 }
 
+dav_template = "_DAV_"
+dav_sources = ["addressbook", "calendar"]
+dav_config = {"peerType": "WebDAV",
+  "sources": {"addressbook": {"sync": "two-way", "backend": "CardDAV", "uri": "addressbook"},
+              "calendar":    {"sync": "two-way", "backend": "CalDAV",  "uri": "calendar"}}
+}
+
 default_modes = [
   # When the user selects Normal Sync, the mode configured in the sync sources will be used.
   # (Each sync source can have its own mode, independent of the other sources.)
   # Could perhaps have a separate "Default" for that, but that might be confusing,
   # and a "Normal Sync" option that enforces two-way sync even for sync sources where only
   # one-way is configured seems like a bad idea anyway.
-  (None, "Normal Sync"),
-  ("slow", "Slow Sync"),
-  ("refresh-from-client", "Refresh From Client"),
-  ("refresh-from-server", "Refresh From Server"),
-  ("one-way-from-client", "One-Way From Client"),
-  ("one-way-from-server", "One-Way From Server"),
+  # (Standard, Reversed, Description)
+  (None, None, "Normal Sync"),
+  ("slow", "slow", "Slow Sync"),
+  ("refresh-from-client", "refresh-from-server", "Refresh From Client"),
+  ("refresh-from-server", "refresh-from-client", "Refresh From Server"),
+  ("one-way-from-client", "one-way-from-server", "One-Way From Client"),
+  ("one-way-from-server", "one-way-from-client", "One-Way From Server"),
 ]
 
 source_modes = [
-  ("disabled", "Disabled"),
-  ("two-way", "Normal Sync"),
-  ("slow", "Slow Sync"),
-  ("refresh-from-client", "Refresh From Client"),
-  ("refresh-from-server", "Refresh From Server"),
-  ("one-way-from-client", "One-Way From Client"),
-  ("one-way-from-server", "One-Way From Server"),
+  # (Standard, Reversed, Description)
+  ("disabled", "disabled", "Disabled"),
+  ("two-way", "two-way", "Normal Sync"),
+  ("slow", "slow", "Slow Sync"),
+  ("refresh-from-client", "refresh-from-server", "Refresh From Client"),
+  ("refresh-from-server", "refresh-from-client", "Refresh From Server"),
+  ("one-way-from-client", "one-way-from-server", "One-Way From Client"),
+  ("one-way-from-server", "one-way-from-client", "One-Way From Server"),
 ]
 
 RESULT_NO_CONNECTION = -2
@@ -245,6 +259,7 @@ class SyncGUI(object):
     self.sync = syncevolution.SyncEvolution(quiet)
     self.dbus = dbus.SessionBus()
     self.server = None
+    self.server_name = None
     self.server_dir = None
     self.server_config = None
     self.server_cookie = None
@@ -261,6 +276,7 @@ class SyncGUI(object):
     self.sync_state = None
     self.sync_runner = None
     self.servers = None
+    self.serverlist = None
     self.selector = None
     self.init_main()
   def main(self):
@@ -292,19 +308,48 @@ class SyncGUI(object):
   def init_main_selector(self):
     if not self.selector is None:
       self.selector.destroy()
+
     self.servers = self.sync.get_servers()
+    self.serverlist = []
+    for server in self.servers:
+      st = server.split("@", 1)
+      if len(st) < 2:
+        st.append("default")
+      # The SyncEvolution authors suggest using the configuration
+      # "consumerReady=0" to hide server configurations that should
+      # not be available for synchronization from GUIs. This is used
+      # to hide the source configuration for CardDAV/CalDAV backends.
+      # However, these same configs also normally get a special name,
+      # "target-config", which is way easier to check, so for now,
+      # I'll just hide these configs based on their name.
+      if st[0] == "target-config":
+        continue
+      self.serverlist.append(st)
     self.selector = hildon.TouchSelector(text = True)
     self.selector.set_hildon_ui_mode("normal")
     self.selector.connect("changed", self.server_selected)
-    for server in self.servers:
-      self.selector.append_text(server)
+    self.init_selector(self.selector)
     self.selector.show()
     self.main_window.add(self.selector)
+  def init_selector(self, selector):
+    names = {}
+    for srv, ctx in self.serverlist:
+      names.setdefault(srv, []).append(ctx)
+    for srv, ctx in self.serverlist:
+      if len(names[srv]) > 1:
+        selector.append_text("%s [%s]" % (srv, ctx))
+      else:
+        selector.append_text(srv)
   def destroy_main(self, window):
     gtk.main_quit()
   def server_selected(self, selector, column):
     row = selector.get_last_activated_row(column)[0]
-    self.server = self.servers[row]
+    server = self.serverlist[row]
+    if server[1] == "default":
+      self.server = server[0]
+    else:
+      self.server = "%s@%s" % (server[0], server[1])
+    self.server_name = server[0]
     self.server_dir = self.sync.get_server_dir(self.server)
     self.server_config = self.sync.get_server_config(self.server)
     self.load_cookie()
@@ -327,8 +372,7 @@ class SyncGUI(object):
     window.set_edit_toolbar(toolbar)
     selector = hildon.TouchSelector(text = True)
     window.add(selector)
-    for server in self.servers:
-      selector.append_text(server)
+    self.init_selector(selector)
     selector.set_column_selection_mode(hildon.TOUCH_SELECTOR_SELECTION_MODE_MULTIPLE)
     selector.unselect_all(0)
     toolbar.connect("button-clicked", self.server_delete_clicked, window, selector)
@@ -356,8 +400,33 @@ class SyncGUI(object):
       response = gtk.Dialog.run(note)
       note.destroy()
     if response == gtk.RESPONSE_OK:
+      names = {}
+      contexts = {}
+      for srv, ctx in self.serverlist:
+        names.setdefault(srv, []).append(ctx)
+        contexts.setdefault(ctx, []).append(srv)
       for server in servers:
+        # more mangling because we don't have get_selected_rows
+        context = None
+        name = server
+        if server[-1] == "]":
+          cpos = server.find(" [")
+          if cpos >= 0:
+            name = server[:cpos]
+            context = server[cpos+2:-1]
+        if context is None:
+          if len(names[name]) > 0:
+            context = names[name][0]
+        if context is not None and context != "default":
+          server = "%s@%s" % (name, context)
         self.do_delete_server(server)
+        # if all servers in a context were deleted,
+        # delete context as well
+        if context is not None:
+          contexts[context].remove(name)
+          if len(contexts[context]) == 0:
+            self.do_delete_context(context)
+            del contexts[context]
       self.server_delete_close(button, window)
       self.init_main_selector()
   def server_delete_close(self, button, window):
@@ -365,23 +434,51 @@ class SyncGUI(object):
   def do_delete_server(self, server):
     server_dir = self.sync.get_server_dir(server)
     server_cookie = self.load_server_cookie(server_dir)
+    server_config = self.sync.get_server_config(server)
     if not server_cookie is None:
       os.unlink(server_dir + "/.alarmcookie")
       alarm.delete_event(server_cookie)
     self.sync.delete_server(server)
+    if self.sync_is_local(server_config) != 0:
+      self.sync.delete_context(url[9:])
+  def do_delete_context(self, context):
+    if not self.sync.has_contexts():
+      return
+    self.sync.delete_context(context)
 
 ### SERVER CREATION
   def init_server_create(self):
     self.wizard_state = {}
 
-    templates = self.sync.get_templates()
-    templates.sort()
+    has_contexts = self.sync.has_contexts()
+
+    std_backends = self.sync.get_backends()
+
+    std_templates = self.sync.get_templates()
+    if len(std_templates) and std_templates[0] == "template name":
+      del std_templates[0]
+    std_templates.sort()
+
+    templates = []
+    if "SyncEvolution" in std_templates:
+      std_templates.remove("SyncEvolution")
+      templates.append(("SyncEvolution", "Generic SyncML", True))
+    else:
+      templates.append(("default", "Generic SyncML", True))
+
+    if std_backends.has_key("CardDAV") or std_backends.has_key("CalDAV"):
+      templates.append((dav_template, "Generic DAV", True))
+
+    # FIXME: perhaps also check for new ActiveSync template
+
+    for label in std_templates:
+      templates.append((label, template_names.get(label, label.replace("_", " ")), False))
 
     self.wizard_state["templates"] = templates
 
     notebook = gtk.Notebook()
 
-    label = gtk.Label("This wizard will walk you through configuring a new SyncML synchronization service.\n"
+    label = gtk.Label("This wizard will walk you through configuring a new synchronization service.\n"
                       "Tap 'Next' to continue.")
     label.set_line_wrap(True)
     notebook.append_page(label, None)
@@ -397,39 +494,73 @@ class SyncGUI(object):
     vbox.pack_start(caption, False, False, 0)
 
     selector = hildon.TouchSelector(text = True)
-    for label in templates:
+    for name, label, generic in templates:
       selector.append_text(label)
     selector.set_active(0, 0)
     self.wizard_state["template"] = selector
 
-    button = hildon.PickerButton(FINGER_SIZE, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+    button = hildon.PickerButton(FINGER_SIZE, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
+    button.set_alignment(0, 0.5, 0.5, 0.5)
     button.set_title("Template")
     button.set_selector(selector)
     vbox.pack_start(button, False, False, 0)
 
+    if has_contexts:
+      contextname = hildon.Entry(gtk.HILDON_SIZE_AUTO)
+      contextname.set_placeholder("Optional. For advanced configurations.")
+      self.wizard_state["context"] = contextname
+      caption = hildon.Caption(None, "Context", contextname)
+      caption.set_status(hildon.CAPTION_MANDATORY)
+      vbox.pack_start(caption, False, False, 0)
+    else:
+      contextname = None
+      self.wizard_state["context"] = contextname
+
     notebook.append_page(vbox, gtk.Label("Basics"))
 
     # Page 3
     vbox = gtk.VBox(False, 0)
 
+    syncurl = hildon.Entry(gtk.HILDON_SIZE_AUTO)
+    syncurl.set_input_mode(gtk.HILDON_GTK_INPUT_MODE_FULL | gtk.HILDON_GTK_INPUT_MODE_DICTIONARY)
+    self.wizard_state["syncurl"] = syncurl
+    caption = hildon.Caption(None, "Sync URL", syncurl)
+    vbox.pack_start(caption, False, False, 0)
+
     username = hildon.Entry(gtk.HILDON_SIZE_AUTO)
+    username.set_input_mode(gtk.HILDON_GTK_INPUT_MODE_FULL | gtk.HILDON_GTK_INPUT_MODE_DICTIONARY)
     self.wizard_state["user"] = username
-    caption = hildon.Caption(None, "Username", username)
+    caption = hildon.Caption(None, "Account", username)
     vbox.pack_start(caption, False, False, 0)
 
     password = hildon.Entry(gtk.HILDON_SIZE_AUTO)
-    password.set_visibility(False)
+    password.set_input_mode(gtk.HILDON_GTK_INPUT_MODE_FULL | gtk.HILDON_GTK_INPUT_MODE_INVISIBLE)
+#    password.set_visibility(False)
     self.wizard_state["pass"] = password
     caption = hildon.Caption(None, "Password", password)
     vbox.pack_start(caption, False, False, 0)
 
-    notebook.append_page(vbox, gtk.Label("Credentials"))
+    notebook.append_page(vbox, gtk.Label("Server access"))
 
     # Page 4
+    area = hildon.PannableArea()
+    area.set_size_request_policy(hildon.SIZE_REQUEST_MINIMUM)
+    vbox = gtk.VBox(False, 0)
+    area.add_with_viewport(vbox)
+    self.wizard_state["dbvbox"] = vbox
+    self.wizard_state["dbarea"] = area
+
+    label = gtk.Label("Database configuration is not currently available for this service type. SyncEvolution will use automatic settings.")
+    label.set_line_wrap(True)
+    self.wizard_state["dblabel"] = label
+    # label will be connected to area on demand.
+
+    notebook.append_page(area, gtk.Label("Server databases"))
+
+    # Page 5
     vbox = gtk.VBox(False, 0)
 
-    label = gtk.Label("Service setup is complete. Tap 'Finish' to save or discard the settings by tapping outside the wizard.\n")
-#                      "Tap 'Advanced settings' to modify the advanced service settings.")
+    label = gtk.Label("Service setup is complete. Tap 'Finish' to save or discard the settings by tapping outside the wizard.")
     label.set_line_wrap(True)
     vbox.pack_start(label, False, False, 0)
 
@@ -445,32 +576,207 @@ class SyncGUI(object):
 
     if response == hildon.WIZARD_DIALOG_FINISH:
       template_idx = selector.get_active(0)
-      self.sync.create_server(servername.get_text(),
-        templates[template_idx],
-        username.get_text(),
-        password.get_text(),
-        {"printChanges": "0"})
+      template, temp_name, temp_custom = templates[template_idx]
+      user_name = username.get_text()
+      user_pass = password.get_text()
+
+      server_name = servername.get_text()
+      if has_contexts:
+        context = contextname.get_text()
+        if context == "":
+          context = server_name
+        server = "%s@%s" % (server_name, context)
+      else:
+        server = server_name
+        context = None
+
+      temp_config = self.get_template_config(template)
+      peer_type = self.get_peer_type(temp_config)
+      temp_sources = self.get_template_sources(temp_config)
+
+      std_config = {"printChanges": "0",
+                    "SSLVerifyServer": "0"}
+      config = std_config.copy()
+      config["syncURL"] = syncurl.get_text()
+      if temp_custom:
+        config["WebURL"] = ""
+
+      disabled_sources = []
+      for name, source_url in self.wizard_state["sources"].items():
+        config1 = {}
+        uri = source_url.get_text()
+        if uri == "":
+          disabled_sources.append(name)
+        else:
+          config1["uri"] = uri
+        if template == dav_template:
+          # "Generic DAV" pseudo-template
+          config2 = self.sync.get_source_config(dav_config, name)
+          config1["backend"] = config2["backend"]
+        self.sync.set_source_config(config, name, config1)
+
+      if len(self.wizard_state["sources"]) == 0 and template == dav_template:
+        config["sources"] = dav_config["sources"]
+
+      if peer_type.startswith("SyncML"):
+        # nice and simple SyncML configuration
+        for name in disabled_sources:
+          config1 = self.sync.get_source_config(config, name)
+          config1["sync"] = "disabled"
+        self.sync.create_server(server,
+          template,
+          user_name,
+          user_pass,
+          config)
+      else:
+        # non-SyncML configuration chosen, need to configure
+        # two SyncML configurations talking to each other
+        src_context = context + "-target"
+        src_server = "target-config@" + src_context
+
+        # First, configure "remote" side
+        src_template = template
+        src_config = config
+        src_config["consumerReady"] = "0" # to hide config from GUIs
+        if template == dav_template:
+          # "Generic DAV" pseudo-template
+          for name in dav_config.keys():
+            if name == "sources":
+              continue
+            src_config[name] = dav_config[name]
+          src_template = None
+
+        self.sync.create_server(src_server,
+          src_template,
+          user_name,
+          user_pass,
+          src_config,
+          temp_sources)
+
+        # Now, configure "local" side
+        config = std_config.copy()
+        config["syncURL"] = "local://@" + src_context
+        for key in ["WebURL", "PeerName", "consumerReady"]:
+          if temp_config.has_key[key]:
+            config[key] = temp_config[key]
+        for name in disabled_sources:
+          config1 = {"sync": "disabled"}
+          self.sync.set_source_config(config, name, config1)
+        self.sync.create_server(server,
+          "SyncEvolution Client",
+          "",
+          "",
+          config,
+          temp_sources)
+
       self.init_main_selector()
 
     dialog.destroy()
     self.wizard_state = None
 
   def server_create_page_switch(self, notebook, page, page_num, dialog):
-    dialog.set_response_sensitive(hildon.WIZARD_DIALOG_FINISH, page_num >= 3)
+    dialog.set_response_sensitive(hildon.WIZARD_DIALOG_FINISH, page_num >= 4)
 
+  def server_create_get_template(self):
+    selector = self.wizard_state["template"]
+    templates = self.wizard_state["templates"]
+    template_idx = selector.get_active(0)
+    return templates[template_idx]
   def server_create_page_func(self, notebook, page_num, user_data):
     if page_num == 1:
       servername = self.wizard_state["name"]
-      return len(servername.get_text()) != 0
+      if len(servername.get_text()) != 0:
+        template, temp_name, temp_custom = self.server_create_get_template()
+        temp_config = self.get_template_config(template)
+
+        syncurl = self.wizard_state["syncurl"]
+
+        if temp_custom:
+          syncurl.set_text("")
+        else:
+          syncurl.set_text(temp_config.get("syncURL", ""))
+
+        return True
+      else:
+        return False
+
+    if page_num == 2:
+      syncurl = self.wizard_state["syncurl"]
+      if len(syncurl.get_text()) != 0:
+        template, temp_name, temp_custom = self.server_create_get_template()
+        temp_config = self.get_template_config(template)
+        peer_type = self.get_peer_type(temp_config)
+
+        source_list = self.build_source_list(temp_config)
+
+        vbox = self.wizard_state["dbvbox"]
+        dbarea = self.wizard_state["dbarea"]
+        dblabel = self.wizard_state["dblabel"]
+
+        # avoid problems by hiding stuff while we mess around
+        dbarea.hide_all()
+
+        if self.wizard_state.has_key("sources"):
+          # remove any previously created source fields
+          sources_remote = self.wizard_state["sources"]
+          for name, source_url in sources_remote.items():
+            # the vbox children are the captions (source_url.parent)
+            vbox.remove(source_url.parent)
+          if len(sources_remote) == 0:
+            vbox.remove(dblabel)
+
+        # add source fields
+        sources_remote = {}
+        for name, label in source_list:
+          config = self.sync.get_source_config(temp_config, name)
+          if peer_type.startswith("SyncML"):
+            source_url = hildon.Entry(gtk.HILDON_SIZE_AUTO)
+            source_url.set_placeholder("Disabled")
+            source_url.set_text(config.get("uri", ""))
+            caption = hildon.Caption(None, label + " database", source_url)
+            vbox.pack_start(caption, False, False, 0)
+            sources_remote[name] = source_url
+        self.wizard_state["sources"] = sources_remote
+
+        if len(sources_remote) == 0:
+          vbox.pack_start(dblabel, False, False, 0)
+
+        dbarea.show_all()
+
+        return True
+      else:
+        return False
+
     return True
+  def get_template_config(self, template):
+    if template == dav_template:
+      # "Generic DAV" pseudo-template
+      return dav_config
+    return self.sync.get_template_config(template)
+  def get_template_sources(self, config):
+    return self.sync.get_sources_from_config(config, all=True)
+  def get_peer_type(self, config):
+    peer_type = config.get("peerType")
+    if peer_type is not None:
+      return peer_type
+    peer_client = config.get("PeerIsClient", "0")
+    if peer_client == "1":
+      return "SyncMLClient"
+    return "SyncMLServer"
+  def sync_is_local(self, config):
+    url = config.get("syncURL", "")
+    if url.startswith("local://@"):
+      return 1
+    else:
+      return 0
 
 ### SERVER WINDOW
   def get_server_title(self):
-    return "Sync with " + self.server
+    return "Sync with " + self.server_name
   def get_source_label(self, source):
     return source_names.get(source, source)
-  def build_source_list(self, all=False):
-    sources = self.sync.get_sources_from_config(self.server_config, all=all)
+  def build_source_list(self, config, all=False):
+    sources = self.sync.get_sources_from_config(config, all=all)
     # build source list with labels for the UI
     source_list = []
     # add known sources
@@ -505,8 +811,8 @@ class SyncGUI(object):
 
     if GUI_VERBOSITY > 0:
       self.sync_mode = hildon.TouchSelector(text = True)
-      for name, label in default_modes:
-        self.sync_mode.append_text(label)
+      for source_mode in default_modes:
+        self.sync_mode.append_text(source_mode[2])
       self.sync_mode.set_active(0, 0)
 
       button = hildon.PickerButton(FINGER_SIZE, hildon.BUTTON_ARRANGEMENT_HORIZONTAL)
@@ -514,7 +820,7 @@ class SyncGUI(object):
       button.set_selector(self.sync_mode)
       vbox.pack_start(button, False, False, 0)
 
-    source_list = self.build_source_list()
+    source_list = self.build_source_list(self.server_config)
 
     if GUI_VERBOSITY > 1:
       # create checkboxes for them
@@ -564,7 +870,7 @@ class SyncGUI(object):
   def edit_server(self, button):
     self.init_server_edit()
   def delete_server(self, button):
-    note = hildon.hildon_note_new_confirmation(self.server_window, "Delete service %s?" % self.server)
+    note = hildon.hildon_note_new_confirmation(self.server_window, "Delete service %s?" % self.server_name)
     response = gtk.Dialog.run(note)
     note.destroy()
     if response == gtk.RESPONSE_OK:
@@ -673,10 +979,15 @@ class SyncGUI(object):
   def server_history_selected(self, selector, column):
     row = selector.get_last_activated_row(column)[0]
     (start_time, code, logf) = self.server_sessions[row]
-    # The first HTML anchor I found in the logs is named H1.
-    # Although it's not the first line of the interesting part of the log,
-    # using it might help the user avoid scrolling past a lot of config cruft.
-    url = "file://" + logf + "#H1"
+    if logf.endswith("syncevolution-log.html"):
+      # For SyncEvolution 1.0+, skip to H2 anchor instead of the
+      # H1 anchor used below, to skip timezone stuff.
+      url = "file://" + logf + "#H2"
+    else:
+      # The first HTML anchor I found in the logs is named H1.
+      # Although it's not the first line of the interesting part of the log,
+      # using it might help the user avoid scrolling past a lot of config cruft.
+      url = "file://" + logf + "#H1"
     self.launch_browser(url)
 
   def destroy_server_history(self, window):
@@ -685,9 +996,21 @@ class SyncGUI(object):
 
 ### SERVER EDITING
   def init_server_edit(self):
+    loc_url = self.server_config.get("syncURL", "")
+    if loc_url.startswith("local://@"):
+      is_local = 1
+      src_server = "target-config@" + loc_url[9:]
+      src_config = self.sync.get_server_config(src_server)
+    else:
+      is_local = 0
+      src_server = self.server
+      src_config = self.server_config
+
+    peer_type = self.get_peer_type(src_config)
+
     dialog = gtk.Dialog()
     dialog.set_transient_for(self.server_window)
-    dialog.set_title("Edit " + self.server)
+    dialog.set_title("Edit " + self.server_name)
     dialog.add_button("Done", gtk.RESPONSE_OK)
 
     area = hildon.PannableArea()
@@ -697,7 +1020,7 @@ class SyncGUI(object):
     dialog.vbox.add(area)
 
     sync_url = hildon.Entry(gtk.HILDON_SIZE_AUTO)
-    sync_url.set_text(self.server_config.get("syncURL", ""))
+    sync_url.set_text(src_config.get("syncURL", ""))
     caption = hildon.Caption(None, "Sync URL", sync_url)
     vbox.pack_start(caption, False, False, 0)
 
@@ -707,81 +1030,140 @@ class SyncGUI(object):
     vbox.pack_start(caption, False, False, 0)
 
     username = hildon.Entry(gtk.HILDON_SIZE_AUTO)
-    username.set_text(self.server_config.get("username", ""))
+    username.set_text(src_config.get("username", ""))
     caption = hildon.Caption(None, "Username", username)
     vbox.pack_start(caption, False, False, 0)
 
     password = hildon.Entry(gtk.HILDON_SIZE_AUTO)
-    password.set_text(self.server_config.get("password", ""))
+    password.set_text(src_config.get("password", ""))
     password.set_visibility(False)
     caption = hildon.Caption(None, "Password", password)
     vbox.pack_start(caption, False, False, 0)
 
-    source_list = self.build_source_list(True)
+    source_list = self.build_source_list(src_config, True)
+
+    # load configs
+    sources_srv_config = {}
+    sources_src_config = {}
+    sources_virtual = {}
+    sources_combined = {}
+    for name, label in source_list:
+      config1 = self.sync.get_source_config(self.server_config, name)
+      config2 = self.sync.get_source_config(src_config, name)
+      sources_srv_config[name] = config1
+      sources_src_config[name] = config2
+      # SyncEvolution 1.2 config
+      type = config1.get("backend")
+      base = config1.get("database")
+      if type is None:
+        # SyncEvolution 1.0 config
+        type = config1.get("type")
+        base = config1.get("evolutionsource")
+        if type is not None:
+          epos = type.find(":")
+          if epos >= 0:
+            type = type[:epos]
+      # check for virtual source
+      if type == "virtual" or \
+         (type is None and base is not None and base.find(",") != -1):
+        if base is not None:
+          vsources = base.split(",")
+          sources_virtual[name] = vsources
+          for vsource in vsources:
+            sources_combined[vsource] = (name, label)
+        else:
+          sources_virtual[name] = []
+
+    # create intuitive ordering of sources
+    source_order = []
+    for nsource in source_list:
+      if nsource in source_order:
+        continue
+      if sources_combined.has_key(nsource[0]):
+        vsource = sources_combined[nsource[0]]
+        if not vsource in source_order:
+          source_order.append(vsource)
+      source_order.append(nsource)
+    source_list = source_order
 
     # server-specific source configuration
     sources_remote = {}
     for name, label in source_list:
-      config = self.sync.get_source_config(self.server_config, name)
-      source_url = hildon.Entry(gtk.HILDON_SIZE_AUTO)
-      source_url.set_text(config.get("uri", ""))
-      caption = hildon.Caption(None, label + " database", source_url)
-      vbox.pack_start(caption, False, False, 0)
-      sources_remote[name] = source_url
+      if not sources_combined.has_key(name):
+        config2 = sources_src_config[name]
+        if not is_local:
+          source_url = hildon.Entry(gtk.HILDON_SIZE_AUTO)
+          source_url.set_text(config2.get("uri", ""))
+          caption = hildon.Caption(None, label + " database", source_url)
+          vbox.pack_start(caption, False, False, 0)
+          sources_remote[name] = source_url
+        else:
+          source_url = hildon.Entry(gtk.HILDON_SIZE_AUTO)
+          source_url.set_text(config2.get("database", ""))
+          caption = hildon.Caption(None, label + " database", source_url)
+          vbox.pack_start(caption, False, False, 0)
+          sources_remote[name] = source_url
 
     # client-specific source configuration
     backends = self.sync.get_backends()
-    sources_local = {}
+    sources_mode = {}
+    sources_base = {}
     for name, label in source_list:
-      config = self.sync.get_source_config(self.server_config, name)
+      config1 = sources_srv_config[name]
+      config2 = sources_src_config[name]
 
-      mode_selector = hildon.TouchSelector(text = True)
-      mode = config.get("sync", "normal")
-      idx = 0
-      count = 0
-      for mode_name, mode_label in source_modes:
-        if mode_name == mode:
-          idx = count
-        mode_selector.append_text(mode_label)
-        count += 1
-      mode_selector.set_active(0, idx)
-
-      button = hildon.PickerButton(FINGER_SIZE, hildon.BUTTON_ARRANGEMENT_VERTICAL)
-      button.set_alignment(0, 0.5, 0.5, 0.5)
-      button.set_title(label + " synchronization")
-      button.set_selector(mode_selector)
-      vbox.pack_start(button, False, False, 0)
-
-      base_selector = hildon.TouchSelector(text = True)
-      base_list = []
-      base = config.get("evolutionsource")
-      idx = 0
-      count = 0
-      for backend_id, bases in backends.items():
-        backend = backend_aliases.get(backend_id, backend_id)
-        if backend != name:
-          continue
-        for base_name, base_uri, default in bases:
-          if base is None:
-            if default:
-              idx = count
-              base_uri = None # if the value was unset, leave unset if the user keeps the setting...
-          elif base == base_uri:
+      if not sources_combined.has_key(name):
+        mode_selector = hildon.TouchSelector(text = True)
+        mode = config1.get("sync", "normal")
+        idx = 0
+        count = 0
+        for source_mode in source_modes:
+          if mode == source_mode[is_local]:
             idx = count
-          base_list.append((backend, base_uri))
-          base_selector.append_text(calendar_names.get(base_name, base_name))
+          mode_selector.append_text(source_mode[2])
           count += 1
-      base_selector.set_active(0, idx)
+        mode_selector.set_active(0, idx)
 
-      # there's only one contacts database, so we don't have to show a button for that
-      if count > 1:
         button = hildon.PickerButton(FINGER_SIZE, hildon.BUTTON_ARRANGEMENT_VERTICAL)
         button.set_alignment(0, 0.5, 0.5, 0.5)
-        button.set_title(label + " source")
-        button.set_selector(base_selector)
+        button.set_title(label + " synchronization")
+        button.set_selector(mode_selector)
         vbox.pack_start(button, False, False, 0)
+        sources_mode[name] = mode_selector
 
-      sources_local[name] = (mode_selector, base_selector, base_list)
+      if not sources_virtual.has_key(name):
+        base_selector = hildon.TouchSelector(text = True)
+        base_list = []
+        base = config1.get("database") # SyncEvolution 1.2
+        if base is None:
+          base = config1.get("evolutionsource") # SyncEvolution 1.0
+        idx = 0
+        count = 0
+        for backend_id, bases in backends.items():
+          backend = backend_aliases.get(backend_id, backend_id)
+          if backend != name:
+            continue
+          for base_name, base_uri, default in bases:
+            if base is None:
+              if default:
+                idx = count
+                base_uri = None # if the value was unset, leave unset if the user keeps the setting...
+            elif base == base_uri:
+              idx = count
+            base_list.append((backend, base_uri))
+            base_selector.append_text(calendar_names.get(base_name, base_name))
+            count += 1
+        base_selector.set_active(0, idx)
+
+        # there's only one contacts database, so we don't have to show a button for that
+        if count > 1:
+          button = hildon.PickerButton(FINGER_SIZE, hildon.BUTTON_ARRANGEMENT_VERTICAL)
+          button.set_alignment(0, 0.5, 0.5, 0.5)
+          button.set_title(label + " source")
+          button.set_selector(base_selector)
+          vbox.pack_start(button, False, False, 0)
+
+        sources_base[name] = (base_selector, base_list)
 
     dialog.show_all()
     response = dialog.run()
@@ -793,13 +1175,34 @@ class SyncGUI(object):
       config["password"] = password.get_text()
       for name, label in source_list:
         source_config = {}
-        source_url = sources_remote[name]
-        mode_selector, base_selector, base_list = sources_local[name]
-        source_config["uri"] = source_url.get_text()
-        idx = mode_selector.get_active(0)
-        source_config["sync"] = source_modes[idx][0]
-        idx = base_selector.get_active(0)
-        source_config["evolutionsource"] = base_list[idx][1]
+        if not sources_combined.has_key(name) and sources_remote.has_key(name):
+          source_url = sources_remote[name]
+          if not is_local:
+            source_config["uri"] = source_url.get_text()
+          else:
+            source_config["database"] = source_url.get_text()
+        self.sync.set_source_config(config, name, source_config)
+      if is_local:
+        self.sync.configure_server(src_server, config)
+        config = {}
+      # set WebURL in both configs, for good measure
+      config["WebURL"] = web_url.get_text()
+      for name, label in source_list:
+        source_config = {}
+        if not sources_combined.has_key(name):
+          mode_selector = sources_mode[name]
+          idx = mode_selector.get_active(0)
+          source_config["sync"] = source_modes[idx][is_local]
+        if not sources_virtual.has_key(name):
+          base_selector, base_list = sources_base[name]
+          idx = base_selector.get_active(0)
+          # SyncEvolution 1.2 renamed the property "evolutionsource"
+          # to "database" (thus we needed to check for both above).
+          # However, we can still configure using just the old name,
+          # and SyncEvolution will transparently map it to the new name.
+          # That way we don't need version checks here...
+          if idx >= 0:
+            source_config["evolutionsource"] = base_list[idx][1]
         self.sync.set_source_config(config, name, source_config)
       self.sync.configure_server(self.server, config)
 
@@ -816,9 +1219,10 @@ class SyncGUI(object):
 ### SYNCHRONIZATION PROGRESS
   def synchronize_start(self, mode = None):
     # get parameters
+    is_local = self.sync_is_local(self.server_config)
     if mode is None and not self.sync_mode is None:
       idx = self.sync_mode.get_active(0)
-      mode = default_modes[idx][0]
+      mode = default_modes[idx][is_local]
     sources = None
     if not self.sync_sources is None:
       sources = []
@@ -834,7 +1238,7 @@ class SyncGUI(object):
     self.sync_bar = gtk.ProgressBar()
     self.sync_bar.set_text("Synchronizing...")
 
-    self.sync_dialog = hildon.hildon_note_new_cancel_with_progress_bar(self.server_window, "Synchronizing with " + self.server, self.sync_bar)
+    self.sync_dialog = hildon.hildon_note_new_cancel_with_progress_bar(self.server_window, "Synchronizing with " + self.server_name, self.sync_bar)
     self.sync_dialog.connect("response", self.synchronize_cancel)
     # I don't seem able to trigger "close" or "delete_event", but just in case anyone can...
     self.sync_dialog.connect("close", self.synchronize_close)
@@ -900,9 +1304,10 @@ class SyncGUI(object):
     return True
 
   def synchronize_recover(self, title):
+    is_local = self.sync_is_local(self.server_config)
     mode_selector = hildon.TouchSelector(text = True)
-    for mode_name, mode_label in default_modes[1:4]:
-      mode_selector.append_text(mode_label)
+    for source_mode in default_modes[1:4]:
+      mode_selector.append_text(source_mode[2])
     dialog = hildon.PickerDialog(self.server_window)
     dialog.set_title(title)
     dialog.set_selector(mode_selector)
@@ -911,7 +1316,7 @@ class SyncGUI(object):
     mode = None
     if response == gtk.RESPONSE_OK:
       idx = mode_selector.get_active(0)
-      mode = default_modes[idx+1][0]
+      mode = default_modes[idx+1][is_local]
     dialog.destroy()
     if not mode is None:
       self.synchronize_start(mode)
@@ -944,12 +1349,13 @@ class SyncCLI(object):
     elif status["result"] == RESULT_ABORTED:
       self.osso_note.system_note_infoprint("Synchronization aborted by user")
     elif status["result"] == RESULT_NO_CONNECTION:
-      self.osso_note.system_note_infoprint("Synchronization failed: No connection")
+#      self.osso_note.system_note_infoprint("Synchronization failed: No connection")
+      self.osso_note.system_note_dialog("Synchronization failed: No connection", 'notice')
     else:
       msg = "Synchronization failed"
       if status.has_key("error"):
         msg += ": " + status["error"]
-      self.osso_note.system_note_infoprint(msg)
+      self.osso_note.system_note_dialog(msg, 'error')
     if gtk.main_level() > 0:
       gtk.main_quit()
   def synchronize_cancel(self, sig, frame):
